      real function getlen(at1,at2,x)
c23456789012345678901234567890123456789012345678901234567890123456789012
C----
C---- calculates distance at1 at2
C----
      implicit      none
      integer  at1, at2
      real     x(*),vecx,vecy,vecz

      vecx=x(3*at1-2)-x(3*at2-2)
      vecy=x(3*at1-1)-x(3*at2-1)
      vecz=x(3*at1)-x(3*at2)

      getlen=sqrt(vecx*vecx+vecy*vecy+vecz*vecz)
      return
      end
      real function getlensq(at1,at2,x)
C----
C---- calculates squared distance at1 at2
C----

      implicit      none
      integer       at1, at2
      real*4        x(*),vecx,vecy,vecz

      vecx=x(3*at1-2)-x(3*at2-2)
      vecy=x(3*at1-1)-x(3*at2-1)
      vecz=x(3*at1)-x(3*at2)

      getlensq=(vecx*vecx+vecy*vecy+vecz*vecz)
      return
      end
      subroutine bonded(b1,b2,nbonds,n1,bond1,nbond1)
C----
C---- returns the nbond1 atom numbers in bond1 to which
C---- atom n1 is bonded
C----
      implicit none
      integer i,b1(*),b2(*),nbonds,n1,bond1(*),nbond1

      nbond1=0
      do i=1,nbonds
         if (b1(i).eq.n1) then
            nbond1=nbond1+1
            bond1(nbond1)=b2(i)
         endif
         if (b2(i).eq.n1) then
            nbond1=nbond1+1
            bond1(nbond1)=b1(i)
         endif
      enddo

      return

      end

      real function getlb(at1,at2,at3,at4,x)
C----
C---- returns lower distance bound for at1-at4 if
C---- at1-at2-at3-at4 are all bonded
C----
C---- this is the cis position, with at1,at2,at3,at4 in one plane
C----
      implicit none
      integer at1,at2,at3,at4
      real x1,y1,x4,y4,getlen,getang,l12,l23,l34,a123,a234,x(*),pi

      pi=4.0*atan(1.0)
      l12=getlen(at1,at2,x)
      l23=getlen(at2,at3,x)
      l34=getlen(at3,at4,x)
      a123=getang(at1,at2,at3,x)
      a234=getang(at2,at3,at4,x)
      x1=0.0-l12*cos(pi-a123)
      y1=l12*sin(pi-a123)
      x4=l23+l34*cos(pi-a234)
      y4=l34*sin(pi-a234)
      getlb=((x1-x4)**2)+((y1-y4)**2)
      if (getlb.lt.0.0) write(0,*) 'NEGATIVE LB'
      getlb=sqrt(getlb)

      return

      end
      real function getub(at1,at2,at3,at4,x)
C----
C---- returns upper distance bound for at1-at4 if
C---- at1-at2-at3-at4 are all bonded
C----
C---- this is the trans position, with at1,at2,at3,at4 in one plane
C----
      implicit none
      integer at1,at2,at3,at4
      real x1,y1,x4,y4,getlen,getang,l12,l23,l34,a123,a234,x(*),pi

      pi=4.0*atan(1.0)
      l12=getlen(at1,at2,x)
      l23=getlen(at2,at3,x)
      l34=getlen(at3,at4,x)
      a123=getang(at1,at2,at3,x)
      a234=getang(at2,at3,at4,x)
      x1=0.0-l12*cos(pi-a123)
      y1=l12*sin(pi-a123)
      x4=l23+l34*cos(pi-a234)
      y4=0.0-l34*sin(pi-a234)
      getub=((x1-x4)**2)+((y1-y4)**2)
      if (getub.lt.0.0) write(0,*) 'NEGATIVE UB'
      getub=sqrt(getub)

      return

      end
      real function getang(at1,at2,at3,x)
C----
C---- calculates angle (rad) at1 at2 at3
C----
      implicit      none
      integer  at1, at2, at3
      real     x(*),len1,len2,vec1x,vec1y,vec1z,vec2x,vec2y,vec2z,
     &     arg,den

      vec1x=x(3*at1-2)-x(3*at2-2)
      vec1y=x(3*at1-1)-x(3*at2-1)
      vec1z=x(3*at1)-x(3*at2)
      vec2x=x(3*at3-2)-x(3*at2-2)
      vec2y=x(3*at3-1)-x(3*at2-1)
      vec2z=x(3*at3)-x(3*at2)

      len1 = sqrt(vec1x*vec1x + vec1y*vec1y + vec1z*vec1z)
      len2 = sqrt(vec2x*vec2x + vec2y*vec2y + vec2z*vec2z)

      den=len1*len2
      if (den.lt.1.e-10) den=1.e-10
      arg=(vec1x*vec2x + vec1y*vec2y + vec1z*vec2z) / den
      if (arg.gt.1.) arg=1.
      if (arg.lt.-1.) arg=-1.

      getang = acos(arg)

      return
      end

      logical function ishbond2(i,j,hbond,nhbo)
C----
C---- determines whether atom i and j form a donor-acceptor
C---- combination
C----
      implicit none
      integer i,j,k,hbond(2,*),nhbo
      ishbond2=.false.

      do k=1,nhbo
         if ((i.eq.hbond(1,k).and.j.eq.hbond(2,k)).or.
     &        (j.eq.hbond(1,k).and.i.eq.hbond(2,k))) ishbond2=.true.
      enddo

      return

      end
      logical function ishbond(i,j,donacc,x,hbdis)
C----
C---- determines whether atom i and j form a donor-acceptor
C---- combination
C----
      implicit none
      integer i,j,donacc(*)
      real x(*),getlen,hbdis
      ishbond=.false.

      if ((donacc(i).eq.1.and.(donacc(j).eq.2.or.donacc(j).eq.3)).or.
     &    (donacc(i).eq.3.and.(donacc(j).eq.2.or.donacc(j).eq.3)).or.
     &    (donacc(i).eq.2.and.(donacc(j).eq.1.or.donacc(j).eq.3)).or.
     &    (donacc(i).eq.3.and.(donacc(j).eq.1.or.donacc(j).eq.3))) then
         if (getlen(i,j,x).lt.hbdis) ishbond=.true.
      endif

      return

      end
      logical function ispothbo(i,j,donacc)
C----
C---- determines whether atom i and j form a potential donor-acceptor
C---- combination
C----
      implicit none
      integer i,j,donacc(*)

      ispothbo=.false.

      if ((donacc(i).eq.1.and.(donacc(j).eq.2.or.donacc(j).eq.3)).or.
     &    (donacc(i).eq.3.and.(donacc(j).eq.2.or.donacc(j).eq.3)).or.
     &    (donacc(i).eq.2.and.(donacc(j).eq.1.or.donacc(j).eq.3)).or.
     &    (donacc(i).eq.3.and.(donacc(j).eq.1.or.donacc(j).eq.3)))
     &   ispothbo=.true.

      return

      end
      logical function ispotaadd(i,j,donacc)
C----
C---- determines whether atom i and j form a potential donor-donor
C---- or acceptor-acceptor combination
C----
      implicit none
      integer i,j,donacc(*)

      ispotaadd=.false.

      if ((donacc(i).eq.1.and.donacc(j).eq.1).or.
     &    (donacc(i).eq.2.and.donacc(j).eq.2))
     &   ispotaadd=.true.

      return

      end
      logical function issalt(at1,at2,charge,x,chdis)
C----
C---- becomes true if at1 and at2 form a salt bridge
C----
      implicit none
      integer at1,at2
      real x(*),charge(*),getlen,chdis

      issalt=.false.

      if (getlen(at1,at2,x)*10..lt.chdis) then
         if (charge(at1).lt.0.0.and.charge(at2).gt.0.0) then
            issalt=.true.
         endif
         if (charge(at2).lt.0.0.and.charge(at1).gt.0.0) then
            issalt=.true.
         endif
      endif
      return

      end
      real function getbondub(n1,n2,rad)
C----
C---- returns upper distance bound for atom n1 and n2 to be covalently 
C---- bonded
      
      implicit none
      integer n1,n2
      real rad(2,*)
      
      getbondub=rad(2,n1)+rad(2,n2)

      return

      end
      subroutine getterm(atnam,resid,x,nres,term)
C----
C---- returns in term those atoms that are part of terminal residues
C----
      implicit none
      integer i,j,term(*),resid(*),n1,n2,nres,findatom
      real x(*),getlen,len
      character*5 atnam(*)
C----
C---- if distance between C and N of next residue > 5A : C terminus
C----
      do i=1,nres-1
         len=10.0
         n1=findatom(i,atnam,resid,'    C')
         n2=findatom(i+1,atnam,resid,'    N')
         if (n1.gt.0.and.n2.gt.0) len=getlen(n1,n2,x)
         if (len.gt.0.5) then
            do j=resid(i)+1,resid(i+1)
               term(j)=1
            enddo
         endif
      enddo
C----
C---- and for last protein residue
C----
      do i=nres,1,-1
         n1=findatom(i,atnam,resid,'    C')
         n2=findatom(i,atnam,resid,'   CA')
         if (n1.gt.0.and.n2.gt.0) then
            do j=resid(i)+1,resid(i+1)
               term(j)=1
            enddo
            goto 10
         endif
      enddo
 10   continue
C----
C---- if distance between C and N of previous residue > 5A : N terminus
C----
      do i=2,nres
         len=10.0
         n1=findatom(i-1,atnam,resid,'    C')
         n2=findatom(i,atnam,resid,'    N')
         if (n1.gt.0.and.n2.gt.0) len=getlen(n1,n2,x)
         if (len.gt.0.5) then
            do j=resid(i)+1,resid(i+1)
               term(j)=-1
            enddo
         endif
      enddo
C----
C---- and for first protein residue
C----
      do i=1,nres
         n1=findatom(i,atnam,resid,'    C')
         n2=findatom(i,atnam,resid,'   CA')
         if (n1.gt.0.and.n2.gt.0) then
            do j=resid(i)+1,resid(i+1)
               term(j)=-1
            enddo
            goto 20
         endif
      enddo
 20   continue

      return

      end

      subroutine dodssp(x,nat,atnm,resnm,resnr,exelin,hst,acc,hbo,hbene,
     &     r)
C----
C---- performs a system call in which dssp is run.
C---- returns secondary structure information
C----
C---- original by Daan van Aalten, hacked by Bert de Groot
C----
      implicit none
      include 'params.f'
      integer i,j,k,nat,atcnt,resnr(*),nres,hbo(4,maxat),gvflen,len,pos,
     &     r(*)
      real x(*),acc(*),hbene(4,maxat)
      character*80 exelin,sdum,dummy
      character*200 syslin
      character*5 atnm(*),resnm(*),dumr(maxat)
      character*1 hst(*)
C----
C---- create pdb file
C----
c      write (6,*) 'ENTER DODSSP'

      write(6,1100)
      open (3,file='dssp.pdb',status='UNKNOWN')
      nres=1
      dumr(1)=resnm(1)
      do atcnt=1,nat
         dummy=atnm(atcnt)
         call gvsshl(dummy)
            if (dummy(1:1).ne.'H'.and.dummy(1:2).ne.'O2') then
               if (dummy(1:2).eq.'O1') dummy='O    '
               write(3,1040) dummy,resnm(atcnt)(1:3),resnr(atcnt),
     &       x(3*atcnt-2)*10.0,x(3*atcnt-1)*10.0,x(3*atcnt)*10.0
               if (atcnt.gt.1) then
                  if (resnr(atcnt).ne.resnr(atcnt-1)) then
                     nres=nres+1
                     dumr(nres)=resnm(atcnt)
                  endif
               endif
            end if
      enddo
      close (3)
      do i=1,nres
         hst(i)=' '
      enddo
C----
C---- run dssp
C----
      write (syslin,1010) exelin,' dssp.pdb dssp.out > dssp.stdout'
      call system(syslin)
C----
C---- read the results
C----
      k=0
      open (3,file='dssp.out',status='OLD')
 30   read (3,1000,err=100,end=120) sdum
      if (sdum(1:6).ne.'  #  R') goto 30
 40   continue
      read (3,1000,err=100,end=120) sdum
      k=k+1
      if (sdum(14:14).eq.'!') goto 40
      read(sdum,1080,err=100,end=120) r(k),hst(r(k)),acc(r(k))
      len=gvflen(sdum)
      pos=40
      do j=1,4
         dummy=sdum(pos:len)
         hbo(j,i)=0
         hbene(j,i)=0.0
         if (dummy(5:5).eq.',') then
            read(dummy,1081,err=100,end=120) hbo(j,k),hbene(j,k)
            pos=pos+9
         else if (dummy(6:6).eq.',') then
            read(dummy,1082,err=100,end=120) hbo(j,k),hbene(j,k)
            pos=pos+10
         endif
      enddo
      goto 40

 100  write(0,*) 'Error in DSSP secondary structure calculation'
      write(0,*) 'near line',i
      write(0,*) 'last line read:'
      write(0,1000) sdum
      stop

 120  continue

      write(6,1101) k

C----
C---- do some minor fudging
C----
      do i=1,nres
         if (hst(i).eq.'L') hst(i)='T'
         if (hst(i).eq.'S') hst(i)='T'
         if (hst(i).eq.'C') hst(i)='T'
         if (hst(i).eq.'B') hst(i)=' '
         if (hst(i).eq.'E') hst(i)='S'
         if (hst(i).eq.'I') hst(i)='H'
         if (hst(i).eq.'G') hst(i)='3'
      enddo
      close(3)
c      write (6,*) 'EXIT DODSSP'

      write(syslin,1000) '/bin/rm -f dssp.pdb dssp.out dssp.stdout'
      call system(syslin)

      return

 1000 format (a80)
 1010 format (2a80)
 1040 format ('ATOM         ',a4,a3,'  ',i4,'    ',3f8.3)
 1080 format (5x,i5,6x,a1,18x,f3.0,1x,4(i4,1x,f4.1))
 1081 format (i4,1x,f4.1)
 1082 format (i5,1x,f4.1)
 1090 format (i5)
 1100 format ('starting DSSP calculation...')
 1101 format ('nr of lines read from DSSP file:',i7)
      end

      integer function findatom(n,atnam,resid,name)
C----
C---- returns atom number of atom with 'NAME' in residue n
C----
      implicit none
      integer n,i,resid(*)
      character*5 atnam(*),name

      findatom=0

      do i=resid(n)+1,resid(n+1)
         if (atnam(i).eq.name) then
            findatom=i
            return
         endif
      enddo

#     if defined (DEBUG)
         write(0,*) 'Warning: atom ',name,' not found in residue',n
#     endif
      return

      end
      integer function findatom2(n,atnam,resnr,name,nat)
C----
C---- returns atom number of atom with 'NAME' in same residue as atom n
C----
      implicit none
      integer n,i,resnr(*),k,l,nat
      character*5 atnam(*),name

      findatom2=0

      k=max(1,n-50)
      l=min(nat,n+50)
      do i=k,l
         if (resnr(n).eq.resnr(i).and.atnam(i).eq.name) then
            findatom2=i
            return
         endif
      enddo

#     if defined (DEBUG)
         write(0,*) 'Warning: atom ',name,' not found in residue ',
     &     resnr(n)
#     endif

      return

      end
      real function davran (idum)
C----
C---- random number generator from numerical recipies
C----
      implicit none
      integer idum,ia,im,iq,ir,mask
      real am

      parameter (ia=16807,im=2147483647,am=1.0/im,
     &     iq=127773,ir=2836,mask=123459876)
      integer k

      idum=xor(idum,mask)
      k=idum/iq
      idum=ia*(idum-k*iq)-ir*k
      if (idum.lt.0) idum=idum+im
      davran=am*idum
      idum=xor(idum,mask)
      return
      end

      logical function had(n1,n2,npair,ihad)
C----
C---- returns true if pair n1,n2 is present in pairlist
C----
C---- beware: n1 and n2 may get swapped!!
C----
      implicit none
      include 'params.f'
      integer i,n1,n2,npair(maxat),ihad(maxat,maxpairs)

      had=.false.
C----
C---- check for trivial errors
C----
      if (n1.eq.n2) then
         had=.true.
         return
      endif
C----
C---- swap atoms: first one should have largest rank number
C----
      if (n2.gt.n1) call swap(n1,n2)
C----
C---- check if the pair is in the existing list
C----
      do i=1,npair(n1)
         if (ihad(n1,i).eq.n2) then
            had=.true.
            return
         endif
      enddo
C----
C---- if we arrive here, it is a new pair: add it to the list
C----
      npair(n1)=npair(n1)+1
      if (npair(n1).gt.maxpairs) then
         write(0,*) 'Problem encountered for atom',n1
         write(0,*) 'Increase parameter MAXPAIRS and recompile'
         stop
      endif
      ihad(n1,npair(n1))=n2


      return

      end

      logical function had1(n1,n2,npair,ihad)
C----
C---- returns true if pair n1,n2 is present in pairlist
C----
C---- beware: n1 and n2 may get swapped!!
C----
      implicit none
      include 'params.f'
      integer i,n1,n2,npair(maxat),ihad(maxat,maxpairs)

      had1=.false.
C----
C---- check for trivial errors
C----
      if (n1.eq.n2) then
         had1=.true.
         return
      endif
C----
C---- swap atoms: first one should have largest rank number
C----
      if (n2.gt.n1) call swap(n1,n2)
C----
C---- check if the pair is in the existing list
C----
      do i=1,npair(n1)
         if (ihad(n1,i).eq.n2) then
            had1=.true.
            return
         endif
      enddo

      return

      end
      logical function had2(i1,i2,in,count)
C----
C---- becomes true if combination i1/i2 is present in list
C----
      implicit none
      include 'params.f'
      integer i,i1,i2
      integer in(2,maxdis),count

      had2=.false.
      do i=1,count
         if (in(1,i).eq.i1.and.in(2,i).eq.i2) had2=.true.
         if (in(1,i).eq.i2.and.in(2,i).eq.i1) had2=.true.
      enddo

      return

      end
      subroutine swap (i,j)

      integer i,j,dum

      dum=i
      i=j
      j=dum

      return

      end
      subroutine rename_atoms(atnam,resnam,nat)
C----
C---- atom names are converted to standard
C----
      implicit none
      integer i,nat
      character*5 atnam(*),resnam(*)

      do i=1,nat
         if (resnam(i)(1:3).eq.'ILE'.and.atnam(i).eq.'  CD1') then
#           if defined (DEBUG)
               write(0,100) atnam(i),'   CD',resnam(i)
#           endif
            atnam(i)='   CD'
         endif
      enddo
#     if defined (DEBUG)
 100  format ('renaming ',A5,' to ',A5,' in residue ',A5)
#     endif

      return

      end

      subroutine getbonded(atnam,resnam,resnr,resid,atrad,nat,nres,
     &               x,b1,b2,npair,ihad,nbond,n13,unit,libdir,type,
     &               sigma,dim,isring)
C----
C---- gathers bonds and angles
C----
      implicit none
      include 'params.f'
      integer i,j,k,l,m,nat,nres,resnr(*),resid(*),dim,
     &     b1(*),b2(*),npair(*),ihad(maxat,maxpairs),
     &     nbond,nbi,nbp,nbs,nbh,n13,unit
      logical had,present,isring(maxat)
      character*5 atnam(*),resnam(*),type(*),at(maxat)
      real x(*),atrad(2,*),len,getlen,getbondub,
     &     sigma(dim,dim,2),ub,lb,usig,lsig
      character*80 libdir
C----
C---- initialize
C----
      nbi=0
      nbp=0
      nbs=0
      nbh=0
      nbond=0
      n13=0
      m=0
C----
C---- find the bonds
C----
      write(unit,200)
      lsig=sigma(1,1,1)
      usig=sigma(1,1,2)
      do i=1,nat
         at(i)=atnam(i)
         call gvsshl(at(i))
      enddo
      do i=1,nat
         do j=resid(resnr(i))+1,i-1
C-------
C------- 1-2 pairs in one residue
C-------
            len=getlen(i,j,x)*10.
            if (len.lt.getbondub(i,j,atrad)) then
               if (.not.(at(i)(1:1).eq.'H'.and.at(j)(1:1).eq.'H')) then
                  if (.not.had(i,j,npair,ihad)) then
                     nbond=nbond+1
                     if (nbond.gt.maxbonds) then
                        write(0,*) 'Increase parameter MAXBONDS',
     &                       ' and compile again'
                        stop
                     endif
                     write(unit,150) i,j,len,len-lsig,len+usig
                     b1(nbond)=i
                     b2(nbond)=j
                     nbi=nbi+1
                  endif
               endif
            endif
         enddo
      enddo
      write(6,220) nbi
C----
C---- peptide bonds
C----
      do i=resid(2)+1,nat
         if (atnam(i).eq.'    N') then
            do j=resid(resnr(i)-1)+1,resid(resnr(i))
               if (atnam(j).eq.'    C') then
                  len=getlen(i,j,x)*10.
                  if (len.lt.getbondub(i,j,atrad)) then
                     if (.not.had(i,j,npair,ihad)) then
                        write(unit,150) i,j,len,len-lsig,len+usig
                        nbond=nbond+1
                        if (nbond.gt.maxbonds) then
                           write(0,*) 'Increase parameter MAXBONDS',
     &                          ' and compile again'
                           stop
                        endif
                        nbp=nbp+1
                        b1(nbond)=i
                        b2(nbond)=j
                     endif
                  endif
               endif
            enddo
         endif
      enddo
      write(6,230) nbp
C----
C---- disulphide bridges
C----
      do k=1,nres
       if (resnam(resid(k+1))(1:3).eq.'CYS') then
        do i=resid(k)+1,resid(k+1)
         if (at(i)(1:1).eq.'S') then
          do l=1,k-1
           if (resnam(resid(l+1))(1:3).eq.'CYS') then
            do j=resid(l)+1,resid(l+1)
             if (at(j)(1:1).eq.'S') then
                len=getlen(i,j,x)*10.
                if (len.lt.getbondub(i,j,atrad)) then
                   if (.not.had(i,j,npair,ihad)) then
                      write(unit,150) i,j,len,len-lsig,len+usig
                      nbond=nbond+1
                      if (nbond.gt.maxbonds) then
                         write(0,*) 'Increase parameter MAXBONDS',
     &                        ' and compile again'
                         stop
                      endif
                      nbs=nbs+1
                      b1(nbond)=i
                      b2(nbond)=j
                   endif
                endif
             endif
            enddo
           endif
          enddo
         endif
        enddo
       endif
      enddo
      write(6,240) nbs
C----
C---- heme links
C----
      do k=1,nres
       if (resnam(resid(k+1))(1:3).eq.'HEM') then
        do l=1,nres
         if (resnam(resid(l+1))(1:3).eq.'HIS') then
          do i=resid(k)+1,resid(k+1)
           if (at(i)(1:2).eq.'FE') then
            do j=resid(l)+1,resid(l+1)
             if (at(j)(1:3).eq.'NE2') then
                len=getlen(i,j,x)*10.
                if (len.lt.getbondub(i,j,atrad)) then
                   if (.not.had(i,j,npair,ihad)) then
                      write(unit,150) i,j,len,len-lsig,len+usig
                      nbond=nbond+1
                      if (nbond.gt.maxbonds) then
                         write(0,*) 'Increase parameter MAXBONDS',
     &                        ' and compile again'
                         stop
                      endif
                      nbh=nbh+1
                      b1(nbond)=i
                      b2(nbond)=j
                   endif
                endif
             endif
            enddo
           endif
          enddo
         endif
        enddo
       endif
      enddo

      write(6,250) nbh
      write(6,260) nbond
C----
C---- define 1-3 constraints from 1-2 constraints
C----
      write(unit,210)
      do k=1,nbond
         do l=1,k-1
            present=.false.
            if (b1(k).eq.b1(l).and.b2(k).ne.b2(l)) then
               i=b2(k)
               j=b2(l)
               m=b1(k)
               present=.true.
            else if (b1(k).eq.b2(l).and.b2(k).ne.b1(l)) then
               i=b2(k)
               j=b1(l)
               m=b1(k)
               present=.true.
            else if (b2(k).eq.b1(l).and.b1(k).ne.b2(l)) then
               i=b1(k)
               j=b2(l)
               m=b2(k)
               present=.true.
            else if (b2(k).eq.b2(l).and.b1(k).ne.b1(l)) then
               i=b1(k)
               j=b1(l)
               m=b2(k)
               present=.true.
            endif
            if (present) then
               if (.not.had(i,j,npair,ihad)) then
                  len=getlen(i,j,x)*10.
                  lsig=sigma(2,2,1)
                  usig=sigma(2,2,2)
                  if (atnam(m).eq.'    C'.and.((atnam(i).eq.'    O'
     &                 .and.atnam(j).eq.'    N').or.(atnam(j).eq.
     &                 '    O'.and.atnam(i).eq.'    N'))) then 
                     lsig=sigma(2,1,1)
                     usig=sigma(2,1,2)
                  endif
                  if (isring(i).and.isring(j)) then
                     lsig=sigma(2,1,1)
                     usig=sigma(2,1,2)
                  endif
                  ub=len+lsig
                  lb=len-usig
                  write(unit,150) i,j,len,lb,ub
                  n13=n13+1
               else
                  write(0,400) i,j
               endif
            endif
         enddo
      enddo

      write(6,270) n13

 150  format (2i5,4f10.4)
 200  format ('#1-2 restricted pairs')
 210  format ('#1-3 restricted pairs')
 220  format ('counted',i7,' intra-residue bonds')
 230  format ('counted',i7,' peptide bonds')
 240  format ('counted',i7,' disulphide bonds')
 250  format ('counted',i7,' histidine-heme bonds')
 260  format ('counted',i7,' bonds in total')
 270  format ('counted',i7,' 1-3 pairs (angles)')
 400  format ('Warning: something strange with pair:',2i5)

      return

      end
      subroutine checklbub(i,j,len,lb,ub)

      implicit none
      integer i,j
      real len,lb,ub

      if (len-ub.gt.0.0001) then
         write(0,100) i,j,len,ub
         stop
      endif
      if (lb-len.gt.0.0001) then
         write(0,110) i,j,len,lb
         stop
      endif

 100  format('distance larger than upper bound:',2i5,2f8.4)
 110  format('distance smaller than lower bound:',2i5,2f8.4)

      return

      end
      subroutine getsecstr1(atnam,type,atrad,resnr,hst,x,sigma,dim,nat,
     &     npair,ihad,unit,libdir,resid,resnam,nhst)
C----
C---- secondary structure restricted pairs
C----
      implicit none
      include 'params.f'
      integer i,j,nat,resnr(*),npair(*),ihad(maxat,maxpairs),unit,nhst,
     &     ncomb,maxcomb,chk,dim,resid(*)
      parameter(maxcomb=500)
      character*1 hst(*)
      character*5 atnam(*),type(*),combtyp1(maxcomb),combtyp2(maxcomb),
     &     resnam(*)
      character*(*) libdir
      logical had,doit
      real x(*),sigma(dim,dim,2),vdwcomb(maxcomb),atrad(2,*),vdwsum,
     &     getvdwsum,lb,ub,marg,len,getlen,lbfun,lsig,usig
C----
C---- determine specific VDW combinations
C----
      call getvdwcomb(combtyp1,combtyp2,vdwcomb,ncomb,maxcomb,libdir)
C----
C---- go through the pairs
C----
      write (unit,100)
      nhst=0
      lsig=0
      usig=0
      marg=sigma(9,3,1)
      do i=1,nat
         if (atnam(i).eq.'    N'.or.atnam(i).eq.'   CA'.or.
     &        atnam(i).eq.'    C') then
            do j=resid(max((resnr(i)-4),1)),i-1
               if (atnam(j).eq.'    N'.or.atnam(j).eq.'   CA'.or.
     &              atnam(j).eq.'    C') then
                  chk=(i+j)/2
                  doit=.false.
                  if ((abs(resnr(i)-resnr(j)).le.1).and.
     &                 ((hst(resnr(i)).eq.'H'.and.hst(resnr(j)).eq.'H'
     &                 .and.hst(resnr(chk)).eq.'H').or.(hst(resnr(i))
     &                 .eq.'S'.and.hst(resnr(j)).eq.'S'.and.
     &                 hst(resnr(chk)).eq.'S'))) then
                     lsig=sigma(7,1,1)
                     usig=sigma(7,1,2)
                     doit=.true.
                  else if ((abs(resnr(i)-resnr(j)).eq.2).and.
     &                 ((hst(resnr(i)).eq.'H'.and.hst(resnr(j)).eq.'H'
     &                    .and.hst(resnr(chk)).eq.'H').or.(hst(resnr(i))
     &                    .eq.'S'.and.hst(resnr(j)).eq.'S'.and.
     &                    hst(resnr(chk)).eq.'S'))) then
                     lsig=sigma(7,2,1)
                     usig=sigma(7,2,2)
                     doit=.true.
                  else if ((abs(resnr(i)-resnr(j)).eq.3).and.
     &                  ((hst(resnr(i)).eq.'H'.and.hst(resnr(j)).eq.'H'
     &                    .and.hst(resnr(chk)).eq.'H').or.(hst(resnr(i))
     &                    .eq.'S'.and.hst(resnr(j)).eq.'S'.and.
     &                    hst(resnr(chk)).eq.'S'))) then
                        lsig=sigma(7,3,1)
                        usig=sigma(7,3,2)
                        doit=.true.
                  else if ((abs(resnr(i)-resnr(j)).eq.4).and.
     &                   ((hst(resnr(i)).eq.'H'.and.hst(resnr(j)).eq.'H'
     &                    .and.hst(resnr(chk)).eq.'H'))) then
                        lsig=sigma(7,4,1)
                        usig=sigma(7,4,2)
                        doit=.true.
                  else if (abs(resnr(i)-resnr(j)).le.1) then
                     lsig=sigma(7,5,1)
                     usig=sigma(7,5,2)
                     doit=.true.
                  else if (abs(resnr(i)-resnr(j)).eq.2) then
                     if (resnam(resid(resnr(j)+2)).eq.'PRO  ') then
                        lsig=sigma(7,8,1)
                        usig=sigma(7,8,2)
                     else
                        lsig=sigma(7,6,1)
                        usig=sigma(7,6,2) 
                     endif
                     doit=.true.
                  else if (abs(resnr(i)-resnr(j)).eq.3) then
                     if (resnam(resid(resnr(j)+2)).eq.'PRO  '.or.
     &                    resnam(resid(resnr(j)+3)).eq.'PRO  ') then
                        lsig=sigma(7,9,1)
                        usig=sigma(7,9,2)
                     else
                        lsig=sigma(7,7,1)
                        usig=sigma(7,7,2)
                     endif
                     doit=.true.
                  endif
                  if (doit) then
                     len=getlen(i,j,x)*10.0
                     if (.not.had(i,j,npair,ihad).and.len.lt.10.) then
                        nhst=nhst+1
                        vdwsum=getvdwsum(combtyp1,combtyp2,vdwcomb,
     &                       ncomb,type(i),type(j),atrad(1,i),
     &                       atrad(1,j))
                        lb=lbfun(i,j,len,lsig,vdwsum,marg)
                        ub=len+usig
                        write(unit,400) i,j,len,lb,ub
                     endif
                  endif
               endif
            enddo
         endif
      enddo
      write (6,300) nhst

 100  format ('#Secondary structure restrictions (sequential)')
 300  format ('counted',i7,' sequential sec. struct. restricted pairs')
 400  format (2i5,4f10.4)

      return

      end
      subroutine getsecstr2(atnam,type,atrad,resnr,hst,x,sigma,dim,nat,
     &     npair,ihad,unit,libdir,resid,resnam,nhst,hbond,nhbo)
C----
C---- more secondary structure restricted pairs
C----
      implicit none
      include 'params.f'
      integer i,j,nat,resnr(*),npair(*),ihad(maxat,maxpairs),unit,nhst,
     &     ncomb,maxcomb,chk,dim,resid(*),hbond(2,maxat),nhbo
      parameter(maxcomb=500)
      character*1 hst(*)
      character*5 atnam(*),type(*),combtyp1(maxcomb),combtyp2(maxcomb),
     &     resnam(*)
      character*(*) libdir
      logical had,doit,ishbond2
      real x(*),sigma(dim,dim,2),vdwcomb(maxcomb),atrad(2,*),vdwsum,
     &     getvdwsum,lb,ub,marg,len,getlen,lbfun,lsig,usig
C----
C---- determine specific VDW combinations
C----
      call getvdwcomb(combtyp1,combtyp2,vdwcomb,ncomb,maxcomb,libdir)
C----
C---- go through the pairs
C----
      write (unit,100)
      marg=sigma(9,3,1)
      usig=0
      lsig=0
      do i=1,nat
         if (atnam(i).eq.'    N'.or.atnam(i).eq.'   CA'.or.
     &        atnam(i).eq.'    C'.or.atnam(i).eq.'   CB'.or.
     &        atnam(i).eq.'    O') then
            do j=resid(max((resnr(i)-4),1)),i-1
               if (atnam(j).eq.'    N'.or.atnam(j).eq.'   CA'.or.
     &              atnam(j).eq.'    C'.or.atnam(j).eq.'   CB'.or.
     &              atnam(j).eq.'    O') then
                  chk=(i+j)/2
                  doit=.false.
                  if (abs(resnr(i)-resnr(j)).le.1.and.
     &                 ((hst(resnr(i)).eq.'H'.and.hst(resnr(j)).eq.'H'
     &                 .and.hst(resnr(chk)).eq.'H').or.(hst(resnr(i))
     &                 .eq.'S'.and.hst(resnr(j)).eq.'S'.and.
     &                 hst(resnr(chk)).eq.'S'))) then
                     lsig=sigma(7,10,1)
                     usig=sigma(7,10,2)
                     doit=.true.
                  else if (abs(resnr(i)-resnr(j)).eq.2.and.
     &                 ((hst(resnr(i)).eq.'H'.and.hst(resnr(j)).eq.'H'
     &                 .and.hst(resnr(chk)).eq.'H').or.(hst(resnr(i))
     &                 .eq.'S'.and.hst(resnr(j)).eq.'S'.and.
     &                 hst(resnr(chk)).eq.'S'))) then
                     lsig=sigma(7,11,1)
                     usig=sigma(7,11,2)
                     doit=.true.
                  else if (abs(resnr(i)-resnr(j)).eq.3.and.
     &                 ((hst(resnr(i)).eq.'H'.and.hst(resnr(j)).eq.'H'
     &                 .and.hst(resnr(chk)).eq.'H').or.(hst(resnr(i))
     &                 .eq.'S'.and.hst(resnr(j)).eq.'S'.and.
     &                 hst(resnr(chk)).eq.'S'))) then
                     lsig=sigma(7,12,1)
                     usig=sigma(7,12,2)
                     doit=.true.
                  else if (abs(resnr(i)-resnr(j)).eq.4.and.
     &                 ((hst(resnr(i)).eq.'H'.and.hst(resnr(j)).eq.'H'
     &                 .and.hst(resnr(chk)).eq.'H'))) then
                     lsig=sigma(7,13,1)
                     usig=sigma(7,13,2)
                     doit=.true.
                  else if (abs(resnr(i)-resnr(j)).le.1) then
                     lsig=sigma(7,14,1)
                     usig=sigma(7,14,2)
                     doit=.true.
                  else if (abs(resnr(i)-resnr(j)).eq.2) then
                     if (resnam(resid(resnr(j)+2)).eq.'PRO  ') then
                        lsig=sigma(7,17,1)
                        usig=sigma(7,17,2)
                     else
                        lsig=sigma(7,15,1)
                        usig=sigma(7,15,2)
                     endif
                     doit=.true.
                  else if (abs(resnr(i)-resnr(j)).eq.3) then
                     if (resnam(resid(resnr(j)+2)).eq.'PRO  '.or.
     &                    resnam(resid(resnr(j)+3)).eq.'PRO  ') then
                        lsig=sigma(7,18,1)
                        usig=sigma(7,18,2)
                     else
                        lsig=sigma(7,16,1)
                        usig=sigma(7,16,2)
                     endif
                     doit=.true.
                  endif
                  if (doit.and..not.ishbond2(i,j,hbond,nhbo)) then
                     len=getlen(i,j,x)*10.0
                     if (.not.had(i,j,npair,ihad).and.len.lt.10.) then
                        nhst=nhst+1
                        vdwsum=getvdwsum(combtyp1,combtyp2,vdwcomb,
     &                       ncomb,type(i),type(j),atrad(1,i),
     &                       atrad(1,j))
                        lb=lbfun(i,j,len,lsig,vdwsum,marg)
                        ub=len+usig
                        write(unit,400) i,j,len,lb,ub
                     endif
                  endif
               endif
            enddo
         endif
      enddo
      write (6,300) nhst

 100  format ('#More secondary structure restrictions (sequential)')
 300  format ('counted',i7,' more sequential sec. struct. restricted ',
     &     'pairs')
 400  format (2i5,4f10.4)

      return

      end

      subroutine getnonbond1(donacc,pol,charge,atrad,type,x,nat,
     &     npair,ihad,rcut,sigma,dim,unit,nnonbond,libdir)
C----
C---- categorize non-bonded pairs
C---- this version does not preserve contacts of starting structure
C----
C---- primitive 'partnerruil'
C----
      implicit none      
      include 'params.f'
      integer i,j,nat,npair(*),ihad(maxat,maxpairs),unit,nnonbond,
     &     donacc(*),maxcomb,ncomb,dim
      parameter(maxcomb=500)
      real charge(*),atrad(2,*),vdwsum,rcut,sigma(dim,dim,2),
     &     getvdwsum,x(*),len,getlen,penal,comp,ub,lb,marg,
     &     vdwcomb(maxcomb),lsig,usig
      logical repulsion,attraction,had,ispothbo
      character*(*) libdir
      character*5 type(*),combtyp1(maxcomb),combtyp2(maxcomb)
      character*1 pol(*)
C----
C---- determine specific VDW combinations
C----
      call getvdwcomb(combtyp1,combtyp2,vdwcomb,ncomb,maxcomb,libdir)
C----
C---- now define non-bonded pairs
C----
      nnonbond=0
      write (unit,100)
      lsig=sigma(9,1,1)
      usig=sigma(9,1,2)
      comp=sigma(9,2,1)
      marg=sigma(9,3,1)
      penal=sigma(9,4,1)
      do i=1,nat
         do j=1,i-1
            len=10.*getlen(i,j,x)
            vdwsum=getvdwsum(combtyp1,combtyp2,vdwcomb,
     &           ncomb,type(i),type(j),atrad(1,i),atrad(1,j))
            if (len.lt.vdwsum+rcut) then
               if (.not.had(i,j,npair,ihad)) then
                  nnonbond=nnonbond+1
                  repulsion=.false.
                  attraction=.false.
                  if (ispothbo(i,j,donacc).or.(charge(i).lt.-0.3.and.
     &                 charge(j).gt.0.3).or.(charge(i).gt.0.3.and.
     &                 charge(j).lt.-0.3).or.(pol(i).eq.'h'.and.
     &                 pol(j).eq.'h'))
     &                 attraction=.true.
                  if (repulsion.and.attraction) then
                     write(0,*) 'error: repulsive attraction',i,j
                     stop
                  endif
                  ub=len+usig
                  if (repulsion) then
                     lb=min(len-marg,vdwsum+penal)
                  else
                     lb=min(len-marg,vdwsum)
                  endif
                  write(unit,300) i,j,len,lb,ub,vdwsum
               endif
            endif
         enddo
      enddo

 100  format ('#non-bonded pairs')
 300  format (2i5,4f10.4)
 310  format (2a5,2f8.4,2l2)

      return

      end
      subroutine getnonbond2(donacc,pol,charge,atrad,x,nat,
     &     npair,ihad,rcut,sigma,dim,unit,nnonbond,hbdis,
     &     chdis,type,libdir,resnr,resnam,atnam,acc,hbond,nhbodum,
     &     nhbo_tight,hst,dsspflg)

      implicit none      
      include 'params.f'
      integer i,nat,npair(*),ihad(maxat,maxpairs),unit,nnonbond,
     &     nsalt,nhbo,nclose,donacc(*),maxcomb,ncomb,
     &     resnr(*),dim,nside,nhbodum,hbond(2,maxat),nhbo_tight,nsheet,
     &     ncon(maxres),icon(maxres,maxpairs,3)
      parameter(maxcomb=500)
      real charge(*),atrad(2,*),rcut,sigma(dim,dim,2),
     &     x(*),hbdis,chdis,vdwcomb(maxcomb),acc(*)
      logical dsspflg
      character*5 type(*),combtyp1(maxcomb),combtyp2(maxcomb),atnam(*),
     &     resnam(*)
      character*1 pol(*),hst(*)
      character*(*) libdir

      nnonbond=0
      write(unit,100)
      do i=1,maxres
         ncon(i)=0
      enddo
C----
C---- determine specific VDW combinations
C----
      call getvdwcomb(combtyp1,combtyp2,vdwcomb,ncomb,maxcomb,libdir)
C----
C---- determine nr of contacts per residue
C----
      call getcon(dim,nat,x,combtyp1,
     &     combtyp2,vdwcomb,ncomb,type,atrad,rcut,resnr,ncon,icon)
C----
C---- salt bridges
C----
      call getsalt(nat,x,sigma,dim,charge,chdis,npair,ihad,type,
     &     atrad,unit,nsalt,combtyp1,combtyp2,vdwcomb,ncomb)
      nnonbond=nnonbond+nsalt
C----
C---- hydrogen bonds
C----
      call gethbonds(dsspflg,unit,sigma,dim,nhbo,nhbo_tight,nhbodum,
     &     hbond,npair,ihad,type,atrad,x,nat,donacc,hbdis,combtyp1,
     &     combtyp2,vdwcomb,ncomb,resnr,ncon,icon)
      nnonbond=nnonbond+nhbo
C----
C---- here for internal side chain restrictions
C----
      call getside(nside,unit,nat,resnr,resnam,sigma,dim,npair,ihad,
     &     combtyp1,combtyp2,vdwcomb,ncomb,type,atrad,x,acc)
      nnonbond=nnonbond+nside
C----
C---- sheet restrictions
C----
      call getsheet(nsheet,unit,sigma,dim,nat,atnam,hst,resnr,x,npair,
     &     ihad,combtyp1,combtyp2,vdwcomb,ncomb,type,atrad)
      nnonbond=nnonbond+nsheet
C----
C---- find other close pairs (distance smaller than vdwsum+rcut)
C----
      call getclose(nclose,sigma,dim,unit,nat,x,combtyp1,
     &     combtyp2,vdwcomb,ncomb,type,atrad,rcut,npair,ihad,atnam,
     &     resnr,ncon,icon,acc)
      nnonbond=nnonbond+nclose

 100  format ('#non-bonded pairs')

      return

      end
      subroutine getother(nat,x,npair,ihad,type,atrad,libdir,unit,
     &     mindis,sigma,dim,noth)
C---- 
C---- find pairs to satisfy minimal nr of distances per atom
C----
      implicit none
      include 'params.f'
      integer i,j,k,l,idum,jdum,npair(*),ihad(maxat,maxpairs),unit,nat,
     &     ndis(maxat),noth,ncomb,maxcomb,iseed,mindis,dim
      parameter(maxcomb=500)
      real x(*),sigma(dim,dim,2),atrad(2,*),vdwcomb(maxcomb),vdwsum,
     &     getvdwsum,davran,len,getlen,lb,ub,lsig,usig
      character*5 type(*),combtyp1(maxcomb),combtyp2(maxcomb)
      character*(*) libdir
      logical had
#     if defined(DEBUG)
      integer mincnt,maxcnt
#     endif

      iseed=93244
      write(unit,100)
C----
C---- first determine how many distances are defined per atom
C----
      do i=1,nat
         ndis(i)=0
      enddo
      do i=1,nat
         do j=1,npair(i)
            ndis(i)=ndis(i)+1
            ndis(ihad(i,j))=ndis(ihad(i,j))+1
         enddo
      enddo

#     if defined (DEBUG)
         mincnt=9999
         maxcnt=0
         do i=1,nat
            mincnt=min(mincnt,ndis(i))
            maxcnt=max(maxcnt,ndis(i))
         enddo
         
         write(6,*) 'MINCNT:',mincnt,' MAXCNT:',maxcnt
#     endif
C----
C---- determine specific VDW combinations
C----
      call getvdwcomb(combtyp1,combtyp2,vdwcomb,ncomb,maxcomb,libdir)
C----
C---- now add (random) restraints if necessary
C----
      noth=0
      mindis=min(mindis,nat-10)
      lsig=sigma(10,1,1)
      usig=sigma(10,1,2)
      do i=1,nat
         if (ndis(i).lt.mindis) then
            l=ndis(i)+1
            do k=l,mindis
 900           j=int(davran(iseed)*nat)+1
               if (j.eq.i.or.j.lt.1.or.j.gt.nat) goto 900
               idum=i
               jdum=j
               if (.not.had(idum,jdum,npair,ihad)) then
                  noth=noth+1
                  len=getlen(i,j,x)*10.
                  vdwsum=getvdwsum(combtyp1,combtyp2,vdwcomb,ncomb,
     &                 type(i),type(j),atrad(1,i),atrad(1,j))
                  lb=max(vdwsum,len-lsig)
                  ub=len+usig
                  write(unit,400) idum,jdum,len,lb,ub
                  ndis(i)=ndis(i)+1
                  ndis(j)=ndis(j)+1
               else
                  goto 900
               endif
            enddo
         endif
      enddo

#     if defined (DEBUG)
         do i=1,nat 
            ndis(i)=0
         enddo
         
         do i=1,nat
            do j=1,npair(i)
               ndis(i)=ndis(i)+1
               ndis(ihad(i,j))=ndis(ihad(i,j))+1
            enddo
         enddo
         
         mincnt=9999
         maxcnt=0
         do i=1,nat
            mincnt=min(mincnt,ndis(i))
            maxcnt=max(maxcnt,ndis(i))
         enddo
         
         write(6,*) 'MINCNT:',mincnt,' MAXCNT:',maxcnt
#     endif

      write(6,300) noth

 100  format ('#other restrictions')
 300  format ('counted',i7,' more restrictions')
 400  format (2i5,4f10.4)


      return

      end
      real function getvdwsum(types1,types2,vdwdis,ncomb,typ1,typ2,
     &     rad1,rad2)
C----
C---- returns vdwsum for combination typ1&typ2
C----
      implicit none
      integer i,ncomb
      character*5 typ1,typ2,types1(*),types2(*)
      real vdwdis(*),rad1,rad2
      
      getvdwsum=0.0
      do i=1,ncomb
         if ((typ1.eq.types1(i).and.typ2.eq.types2(i)).or.
     &        (typ2.eq.types1(i).and.typ1.eq.types2(i))) then
            getvdwsum=vdwdis(i)
            return
         endif
      enddo
      
      if (abs(getvdwsum).lt.0.01) getvdwsum=rad1+rad2

      return

      end

      real function lbfun(i,j,len,sigma,vdwsum,marg) 
      implicit none
      integer i,j
      real len,sigma,sig,vdwsum,marg
      
      sig=sigma/3.
      if (len-sigma.gt.vdwsum) then
         lbfun=len-sigma
      else if (len.gt.vdwsum+marg) then
         lbfun=vdwsum
      else if (len.gt.vdwsum) then
         lbfun=vdwsum-marg
      else
         if (len.lt.vdwsum-marg) write(0,100) i,j,len,vdwsum
         lbfun=len-marg
      endif

 100  format('remark: distance between atom',i5,' and',i5,' (',f4.2,
     &     ') smaller than vdwsum (',f4.2,')')

      return
      end

      real function getlsig1(i,j,atnam,sigma,dim)
      implicit none
      integer i,j,dim
      character*5 atnam(*)
      real sigma(dim,dim,2)
      
      if (atnam(i).eq.'    C'.and.atnam(j).eq.'    C') then
         getlsig1=sigma(5,1,1)
      else if (atnam(i).eq.'   CB'.and.atnam(j).eq.'    C') then
         getlsig1=sigma(5,2,1)
      else if (atnam(i).eq.'    N'.and.atnam(j).eq.'    N') then
         getlsig1=sigma(5,3,1)
      else if (atnam(i).eq.'    N'.and.atnam(j).eq.'   CB') then
         getlsig1=sigma(5,4,1)   
      else if (atnam(i).eq.'    O'.and.atnam(j).eq.'    N') then
         getlsig1=sigma(5,5,1)
      else if (atnam(i).eq.'    O'.and.atnam(j).eq.'   CB') then
         getlsig1=sigma(5,6,1)
      else
         getlsig1=sigma(5,14,1)
      endif

      return
      
      end
      real function getusig1(i,j,atnam,sigma,dim)
      implicit none
      integer i,j,dim
      character*5 atnam(*)
      real sigma(dim,dim,2)
      
      if (atnam(i).eq.'    C'.and.atnam(j).eq.'    C') then
         getusig1=sigma(5,1,2)
      else if (atnam(i).eq.'   CB'.and.atnam(j).eq.'    C') then
         getusig1=sigma(5,2,2)
      else if (atnam(i).eq.'    N'.and.atnam(j).eq.'    N') then
         getusig1=sigma(5,3,2)
      else if (atnam(i).eq.'    N'.and.atnam(j).eq.'   CB') then
         getusig1=sigma(5,4,2)   
      else if (atnam(i).eq.'    O'.and.atnam(j).eq.'    N') then
         getusig1=sigma(5,5,2)
      else if (atnam(i).eq.'    O'.and.atnam(j).eq.'   CB') then
         getusig1=sigma(5,6,2)
      else
         getusig1=sigma(5,14,2)
      endif

      return
      
      end

      real function getlsig2(i,j,atnam,resnam,sigma,dim)
      implicit none
      integer i,j,dim
      character*5 atnam(*),resnam(*)
      real sigma(dim,dim,2)
      
      if (atnam(j).eq.'    C'.and.(resnam(j).eq.'PRO  '.or.resnam(j)
     &     .eq.'GLY  ')) then
         getlsig2=sigma(5,13,1)
      else if (resnam(i).eq.'PRO  '.or.resnam(i).eq.'GLY  ') then
         getlsig2=sigma(5,13,1)   
      else if (atnam(i).eq.'    C'.and.atnam(j).eq.'    C') then
         getlsig2=sigma(5,7,1)
      else if (atnam(i).eq.'   CB'.and.atnam(j).eq.'    C') then
         getlsig2=sigma(5,8,1)
      else if (atnam(i).eq.'    N'.and.atnam(j).eq.'    N') then
         getlsig2=sigma(5,9,1)
      else if (atnam(i).eq.'    N'.and.atnam(j).eq.'   CB') then
         getlsig2=sigma(5,10,1)   
      else if (atnam(i).eq.'    O'.and.atnam(j).eq.'    N') then
         getlsig2=sigma(5,11,1)
      else if (atnam(i).eq.'    O'.and.atnam(j).eq.'   CB') then
         getlsig2=sigma(5,12,1)
      else
         getlsig2=sigma(5,14,1)
      endif

      return
      
      end
      real function getusig2(i,j,atnam,resnam,sigma,dim)
      implicit none
      integer i,j,dim
      character*5 atnam(*),resnam(*)
      real sigma(dim,dim,2)
      
      if (atnam(j).eq.'    C'.and.(resnam(j).eq.'PRO  '.or.resnam(j)
     &     .eq.'GLY  ')) then
         getusig2=sigma(5,13,2)
      else if (resnam(i).eq.'PRO  '.or.resnam(i).eq.'GLY  ') then
         getusig2=sigma(5,13,2)   
      else if (atnam(i).eq.'    C'.and.atnam(j).eq.'    C') then
         getusig2=sigma(5,7,2)
      else if (atnam(i).eq.'   CB'.and.atnam(j).eq.'    C') then
         getusig2=sigma(5,8,2)
      else if (atnam(i).eq.'    N'.and.atnam(j).eq.'    N') then
         getusig2=sigma(5,9,2)
      else if (atnam(i).eq.'    N'.and.atnam(j).eq.'   CB') then
         getusig2=sigma(5,10,2)   
      else if (atnam(i).eq.'    O'.and.atnam(j).eq.'    N') then
         getusig2=sigma(5,11,2)
      else if (atnam(i).eq.'    O'.and.atnam(j).eq.'   CB') then
         getusig2=sigma(5,12,2)
      else
         getusig2=sigma(5,14,2)
      endif

      return
      
      end

      subroutine getsalt(nat,x,sigma,dim,charge,chdis,npair,ihad,
     &     type,atrad,unit,nsalt,combtyp1,combtyp2,vdwcomb,ncomb)
      implicit none
      include 'params.f'
      integer i,j,nat,npair(*),ihad(maxat,maxpairs),unit,
     &     nsalt,ncomb,dim
      real charge(*),atrad(2,*),vdwsum,sigma(dim,dim,2),
     &     x(*),len,getlen,chdis,vdwcomb(*),lb,ub,
     &     getvdwsum,marg,lbfun,lsig,usig
      logical had,issalt
      character*5 type(*),combtyp1(*),combtyp2(*)

      nsalt=0
      write(unit,400)
      lsig=sigma(8,1,1)
      usig=sigma(8,1,2)
      marg=sigma(9,3,1)
      do i=1,nat
         if (abs(charge(i)).gt.0.001) then
            do j=1,i-1
               if (abs(charge(j)).gt.0.001) then
                  if(issalt(i,j,charge,x,chdis)) then
                     if (.not.had(i,j,npair,ihad)) then
                        vdwsum=getvdwsum(combtyp1,combtyp2,vdwcomb,
     &                       ncomb,type(i),type(j),atrad(1,i),
     &                       atrad(1,j))
                        len=getlen(i,j,x)*10.0
                        lb=lbfun(i,j,len,lsig,vdwsum,marg)
                        ub=len+usig
                        write(unit,300) i,j,len,lb,ub
                        nsalt=nsalt+1
                     endif
                  endif
               endif
            enddo
         endif
      enddo
      write (6,500) nsalt

 300  format (2i5,4f10.4)
 400  format ('#Salt bridges')
 500  format ('counted',i7,' salt bridges')

      return

      end
      subroutine gethbonds(dsspflg,unit,sigma,dim,nhbo,nhbo_tight,
     &     nhbodum,hbond,npair,ihad,type,atrad,x,nat,donacc,hbdis,
     &     combtyp1,combtyp2,vdwcomb,ncomb,resnr,ncon,icon)

      implicit none      
      include 'params.f'
      integer i,j,nat,npair(*),ihad(maxat,maxpairs),unit,nhbo,donacc(*),
     &     ncomb,dim,nhbodum,hbond(2,maxat),nhbo_tight,k,resnr(*),
     &     ncon(*),icon(maxres,maxpairs,3),idum,findcon
      real atrad(2,*),vdwsum,sigma(dim,dim,2),x(*),len,getlen,hbdis,
     &     vdwcomb(*),lsig,usig,lb,ub,getvdwsum,marg,lbfun
      logical had,ishbond,dsspflg
      character*5 type(*),combtyp1(*),combtyp2(*)

      marg=sigma(9,3,1)
      if (dsspflg) then
         write (unit,410)
         lsig=sigma(8,2,1)
         usig=sigma(8,2,2)
         nhbo=0
         do k=1,nhbo_tight
            i=hbond(1,k)
            j=hbond(2,k)
            if (abs(resnr(i)-resnr(j)).le.4.or.
     &           findcon(i,j,resnr,ncon,icon).gt.50) then
               if (.not.had(i,j,npair,ihad)) then
                  vdwsum=getvdwsum(combtyp1,combtyp2,vdwcomb,
     &                 ncomb,type(i),type(j),atrad(1,i),atrad(1,j))
                  len=getlen(i,j,x)*10.0
                  lb=lbfun(i,j,len,lsig,vdwsum,marg) 
                  ub=len+usig
                  write(unit,300) i,j,len,lb,ub
                  nhbo=nhbo+1
               endif
            endif
         enddo
         write (6,510) nhbo
         write (unit,411)
         lsig=sigma(8,3,1)
         usig=sigma(8,3,2)
         idum=0
         do k=nhbo_tight+1,nhbodum
            i=hbond(1,k)
            j=hbond(2,k)
            if (abs(resnr(i)-resnr(j)).le.4.or.
     &           findcon(i,j,resnr,ncon,icon).gt.50) then
               if (.not.had(i,j,npair,ihad)) then
                  vdwsum=getvdwsum(combtyp1,combtyp2,vdwcomb,
     &                 ncomb,type(i),type(j),atrad(1,i),atrad(1,j))
                  len=getlen(i,j,x)*10.0
                  lb=lbfun(i,j,len,lsig,vdwsum,marg) 
                  ub=len+usig
                  write(unit,300) i,j,len,lb,ub
                  nhbo=nhbo+1
                  idum=idum+1
               endif
            endif
         enddo
         write (6,511) idum
      else
         write (unit,411)
         lsig=sigma(8,3,1)
         usig=sigma(8,3,2)
         nhbo=0
         do i=1,nat
            do j=1,i-1
               if (ishbond(i,j,donacc,x,hbdis)) then
                  if (.not.had(i,j,npair,ihad)) then
                     vdwsum=getvdwsum(combtyp1,combtyp2,vdwcomb,
     &                    ncomb,type(i),type(j),atrad(1,i),atrad(1,j))
                     len=getlen(i,j,x)*10.0
                     lb=lbfun(i,j,len,lsig,vdwsum,marg) 
                     ub=len+usig
                     write(unit,300) i,j,len,lb,ub
                     nhbo=nhbo+1
                  endif
               endif
            enddo
         enddo
         write (6,512) nhbo
      endif

 300  format (2i5,4f10.4)
 410  format ('#Tight hydrogen bonds')
 411  format ('#Weak hydrogen bonds')
 510  format ('counted',i7,' tight hydrogen bonds')
 511  format ('counted',i7,' weak hydrogen bonds')
 512  format ('counted',i7,' hydrogen bonds')

      return

      end

      subroutine getside(nside,unit,nat,resnr,resnam,sigma,dim,npair,
     &     ihad,combtyp1,combtyp2,vdwcomb,ncomb,type,atrad,x,acc)

      implicit none      
      include 'params.f'
      integer i,j,nat,npair(*),ihad(maxat,maxpairs),unit,
     &     ncomb,resnr(*),dim,nside
      real atrad(2,*),vdwsum,sigma(dim,dim,2),x(*),len,getlen,
     &     vdwcomb(*),lsig,usig,lb,ub,getvdwsum,marg,lbfun,acc(*)
      logical had
      character*5 type(*),combtyp1(*),combtyp2(*),resnam(*)

      nside=0
      write (unit,416)
      marg=sigma(9,3,1)
      do i=1,nat
         do j=1,i-1
            if (resnr(i).eq.resnr(j)) then
               if (acc(i).lt.0.2) then
                  lsig=sigma(8,4,1)
                  usig=sigma(8,4,2)
               else
                  lsig=sigma(8,5,1)
                  usig=sigma(8,5,2)
               endif
               if (.not.had(i,j,npair,ihad)) then
                  vdwsum=getvdwsum(combtyp1,combtyp2,vdwcomb,
     &                 ncomb,type(i),type(j),atrad(1,i),atrad(1,j))
                  len=getlen(i,j,x)*10.
                  lb=lbfun(i,j,len,lsig,vdwsum,marg) 
                  ub=len+usig
                  write(unit,300) i,j,len,lb,ub
                  nside=nside+1
               endif
            endif
         enddo
      enddo

      write (6,516) nside     

 300  format (2i5,4f10.4)
 416  format ('#Side chain restrictions')
 516  format ('counted',i7,' side chain restrictions')

      return

      end

      subroutine getsheet(nsheet,unit,sigma,dim,nat,atnam,hst,resnr,x,
     &     npair,ihad,combtyp1,combtyp2,vdwcomb,ncomb,type,atrad)

      implicit none      
      include 'params.f'
      integer i,j,nat,npair(*),ihad(maxat,maxpairs),unit,
     &     ncomb,resnr(*),dim,nsheet
      real atrad(2,*),vdwsum,sigma(dim,dim,2),x(*),len,getlen,
     &     vdwcomb(*),lsig,usig,lb,ub,getvdwsum,marg,lbfun
      logical had
      character*5 type(*),combtyp1(*),combtyp2(*),atnam(*)
      character*1 hst(*)

      nsheet=0
      write (unit,417)
      lsig=sigma(8,6,1)
      usig=sigma(8,6,2)
      marg=sigma(9,3,1)
      do i=1,nat
         if ((atnam(i).eq.'    N'.or.atnam(i).eq.'   CA'.or.
     &        atnam(i).eq.'    C'.or.atnam(i).eq.'   CB'.or.
     &        atnam(i).eq.'    O').and.hst(resnr(i)).eq.'S') then
            do j=1,i-1
               if ((atnam(j).eq.'    N'.or.atnam(j).eq.'   CA'.or.
     &              atnam(j).eq.'    C'.or.atnam(j).eq.'   CB'.or.
     &              atnam(j).eq.'    O').and.hst(resnr(j)).eq.'S') then
                  len=getlen(i,j,x)*10.0
                  if (len.lt.6.0) then
                     if (.not.had(i,j,npair,ihad)) then
                        len=getlen(i,j,x)*10.
                        vdwsum=getvdwsum(combtyp1,combtyp2,vdwcomb,
     &                     ncomb,type(i),type(j),atrad(1,i),atrad(1,j))
                        lb=lbfun(i,j,len,lsig,vdwsum,marg) 
                        ub=len+usig
                        write(unit,300) i,j,len,lb,ub
                        nsheet=nsheet+1
                     endif
                  endif
               endif
            enddo
         endif
      enddo
      write (6,517) nsheet

 300  format (2i5,4f10.4)
 417  format ('#Sheet restrictions')
 517  format ('counted',i7,' sheet restrictions')

      return

      end

      subroutine getclose(nclose,sigma,dim,unit,nat,x,combtyp1,
     &     combtyp2,vdwcomb,ncomb,type,atrad,rcut,npair,ihad,atnam,
     &     resnr,ncon,icon,acc)

      implicit none      
      include 'params.f'
      integer i,j,nat,npair(*),ihad(maxat,maxpairs),unit,findcon,
     &     nclose,ncomb,dim,ncon(*),icon(maxres,maxpairs,3),resnr(*)
      real atrad(2,*),vdwsum,rcut,sigma(dim,dim,2),x(*),len,getlensq,
     &     vdwcomb(*),lsig,usig,lb,ub,getvdwsum,marg,lbfun,dum,acc(*)
      logical had
      character*5 type(*),combtyp1(*),combtyp2(*),atnam(*)
C----
C---- find the pairs
C----
      dum=(4.+rcut)**2
      write (unit,430)
      marg=sigma(9,3,2)
      nclose=0
      do i=1,nat
         do j=1,i-1
            len=getlensq(i,j,x)*100.
            if (len.lt.dum) then
               if (resnr(i)-resnr(j).le.3) then
                  if (acc(i).lt.0.2.and.acc(j).lt.0.2) then
                     lsig=sigma(8,7,1)
                     usig=sigma(8,7,2)
                  else
                     lsig=sigma(8,8,1)
                     usig=sigma(8,8,2)                     
                  endif
               else if (findcon(i,j,resnr,ncon,icon).gt.50) then
                  if (acc(i).lt.0.2.and.acc(j).lt.0.2) then
                     lsig=sigma(8,9,1)
                     usig=sigma(8,9,2)
                  else
                     lsig=sigma(8,10,1)
                     usig=sigma(8,10,2)                     
                  endif
               else
                  lsig=sigma(8,11,1)
                  usig=sigma(8,11,2)
               endif
               len=sqrt(len)
               vdwsum=getvdwsum(combtyp1,combtyp2,vdwcomb,ncomb,
     &              type(i),type(j),atrad(1,i),atrad(1,j))
               if ((len-vdwsum).lt.rcut) then
                  if (.not.had(i,j,npair,ihad)) then
                     lb=lbfun(i,j,len,lsig,vdwsum,marg)
                     ub=len+usig
                     write(unit,300) i,j,len,lb,ub
                     nclose=nclose+1
                  endif
               endif
            endif
         enddo
      enddo

      write (6,530) nclose

 300  format (2i5,4f10.4)
 430  format ('#Other close pairs')
 530  format ('counted',i7,' other close pairs')

      return

      end
      subroutine getcon(dim,nat,x,combtyp1,
     &     combtyp2,vdwcomb,ncomb,type,atrad,rcut,resnr,ncon,icon)

      implicit none      
      include 'params.f'
      integer i,j,k,l,m,n,o,nat,ncomb,dim,ncon(*),
     &     icon(maxres,maxpairs,3),resnr(*),nwin,nres,findcon2
      real atrad(2,*),vdwsum,rcut,x(*),len,getlensq,vdwcomb(*),
     &     getvdwsum,dum,rc
      character*5 type(*),combtyp1(*),combtyp2(*)

C----
C---- determine nr of contacts per residue pair
C----
      rc=2.
      dum=(4.+rc)**2
      do i=1,nat
         do j=1,i-1
            if ((resnr(i)-resnr(j)).gt.4) then
               len=getlensq(i,j,x)*100.
               if (len.lt.dum) then
                  len=sqrt(len)
                  vdwsum=getvdwsum(combtyp1,combtyp2,vdwcomb,ncomb,
     &                 type(i),type(j),atrad(1,i),atrad(1,j))
                  if (len.lt.(vdwsum+rc))
     &                 call addcon(resnr(i),resnr(j),ncon,icon)
               endif
            endif
         enddo
      enddo
C----
C---- do window averaging
C----
      nwin=3
      nres=resnr(nat)
      do i=1,nres
         do j=1,ncon(i)
            m=icon(i,j,1)
            icon(i,j,3)=0
            do k=max(1,i-nwin),min(nres,i+nwin)
               do l=max(1,m-nwin),min(nres,m+nwin)
                  icon(i,j,3)=icon(i,j,3)+findcon2(k,l,ncon,icon)
               enddo
            enddo
         enddo
      enddo
C----
C---- do triangle averaging
C----
c      do i=1,nres
c         do j=1,ncon(i)
c            k=icon(i,j,1)
c            do l=1,ncon(i)
c               m=icon(i,l,1)
c               do n=max(1,k-nwin),min(nres,k+nwin)
c                  do o=max(1,m-nwin),min(nres,m+nwin)
c                     icon(i,j,3)=icon(i,j,3)+findcon2(n,o,ncon,icon)
c                  enddo
c               enddo
c            enddo
c         enddo
c      enddo

c      do i=1,nres
c         do j=1,ncon(i)
c            write(98,*) i,j,icon(i,j,1),icon(i,j,2),icon(i,j,3)
c         enddo
c      enddo
         

      return

      end

      subroutine addcon(i,j,ncon,icon)

      implicit none

      include 'params.f'

      integer i,j,k,ncon(*),icon(maxres,maxpairs,3)

      logical found1,found2

      found1=.false.
      do k=1,ncon(i)
         if (j.eq.icon(i,k,1)) then
            icon(i,k,2)=icon(i,k,2)+1
            found1=.true.
         endif
      enddo

      found2=.false.
      do k=1,ncon(j)
         if (i.eq.icon(j,k,1)) then
            icon(j,k,2)=icon(j,k,2)+1
            found2=.true.
         endif
      enddo

      if (found1.neqv.found2) then
         write (0,*) 'error - internal inconsistency in findcon (bug)'
         stop
      endif

      if (.not.found1) then
         ncon(i)=ncon(i)+1
         if (ncon(i).gt.maxpairs) then
            write(0,*) 'increase parameter MAXPAIRS'
            stop
         endif
         icon(i,ncon(i),1)=j
         icon(i,ncon(i),2)=1
         ncon(j)=ncon(j)+1
         if (ncon(j).gt.maxpairs) then
            write(0,*) 'increase parameter MAXPAIRS'
            stop
         endif
         icon(j,ncon(j),1)=i
         icon(j,ncon(j),2)=1
      endif

      return

      end

      integer function findcon(i,j,resnr,ncon,icon)

      include 'params.f'

      integer i,j,k,ir,jr,resnr(*),ncon(*),icon(maxres,maxpairs,3)

      findcon=0
      ir=resnr(i)
      jr=resnr(j)
      do k=1,ncon(ir)
         if (jr.eq.icon(ir,k,1)) findcon=icon(ir,k,3)
      enddo
      
      return

      end

      integer function findcon2(i,j,ncon,icon)

      include 'params.f'

      integer i,j,k,ncon(*),icon(maxres,maxpairs,3)

      findcon2=0
      do k=1,ncon(i)
         if (j.eq.icon(i,k,1)) findcon2=icon(i,k,2)
      enddo
      
      return

      end


